#!/usr/bin/env fades

import argparse
import re
import subprocess

from urllib import request, parse

import requests  # fades
from bs4 import BeautifulSoup  # fades

SITE = "www.argenteam.net"
BASE_URL = "http://%s" % (SITE,)
USER_AGENT = "Mozilla/5.0 (X11; Linux i686; rv:10.0) Gecko/20100101 Firefox/10.0"

RE_MAGNET = re.compile(r'^magnet:\?xt')
RE_SUBTITLE = re.compile(r'^/subtitle')

# the versions that we consider useful to download, in order (better 1st)
VERSIONS = [
    re.compile(r"HDTV.*x264.*1080p"),
    re.compile(r".*x264.*1080p"),
    re.compile(r".*H.264.*1080p"),
    re.compile(r".*1080p"),
    re.compile(r"\sHDTV.*720p"),
    re.compile(r".*H264.*720p"),
    re.compile(r".*720p"),
    re.compile(r".*HDTV XviD"),
    re.compile(r".*BDRip x264  AAC"),
]


def get_page(url):
    """Get a page content."""
    response = requests.get(url, headers={'User-Agent': USER_AGENT})
    return response.text


def sort_by_tag(info):
    """Get a release to download."""
    tag = info[0]
    for pos, regex in enumerate(VERSIONS):
        if regex.match(tag):
            break
    return pos


def get_data(url):
    """Get the movie/episode data."""
    page = get_page(url)
    bs = BeautifulSoup(page, "html.parser")
    ep_list = bs.find('div', attrs={'class': 'episode-list'})
    episode_regex = re.compile('^episode-item.*')
    episodes = ep_list.findAll('div', attrs={'class': episode_regex})
    ep_urls = []
    for ep in episodes:
        anchor = ep.find('a')
        if anchor is not None:
            ep_urls.append(parse.urljoin(BASE_URL, anchor['href']))

    all_episodes = []
    for ep_url in ep_urls:
        print("  parsing", ep_url)
        ep_page = get_page(ep_url)
        ep_bs = BeautifulSoup(ep_page, "html.parser")
        # import pdb;pdb.set_trace()
        releases = ep_bs.find('div', attrs={'class': 'releases'})
        episode_info = ep_bs.find('div', attrs={'class': "episode-info"})
        name = episode_info.find('h1').text.strip()

        data = []
        for release_tag in releases.findAll('div', 'release-tags'):
            release_info = release_tag.parent.findNextSibling()

            tag_text = release_tag.contents[1].strip()

            magnet_field = release_info.find('a', href=RE_MAGNET)
            magnet_link = None if magnet_field is None else magnet_field['href']

            sub_field = release_info.find('a', href=RE_SUBTITLE)
            sub_link = None if sub_field is None else sub_field['href']

            data.append((tag_text, magnet_link, sub_link))
        all_episodes.append((name, data))

    return all_episodes


def download_magnet(magnet_link):
    cmd = ['deluge-console', 'add', magnet_link]
    subprocess.run(cmd)


def download_subs(sub_link):
    url = parse.urljoin(BASE_URL, sub_link)
    req = request.Request(url, headers={'User-Agent': USER_AGENT})
    resp = request.urlopen(req)
    filename = parse.urlsplit(resp.geturl()).path.lstrip('/')
    with open(filename, 'wb') as f:
        f.write(resp.read())


def download_episode(data, version, detached, only_subs, only_magnet):
    for tag, magnet_link, sub_link in data:
        if version is not None:
            tag_parts = set(tag.split())
            version_parts = set(version.split())
            if not version_parts.issubset(tag_parts):
                print("    Ignoring, bad version:", repr(tag))
                continue

        if magnet_link is None and not only_subs:
            print("    Ignoring, no magnet link:", repr(tag))
            continue

        if sub_link is None and not only_magnet:
            print("    Ignoring, no subs:", repr(tag))
            continue

        print("Got:", repr(tag))
        if not only_magnet:
            download_subs(sub_link)
        if not only_subs:
            download_magnet(magnet_link)
        return

    if not detached:
        print("Not found!!")
        return

    print("Attempting detached")
    got_subs = got_magnet = False
    for tag, magnet_link, sub_link in data:
        if not got_subs and sub_link is not None:
            print("    Subs from", repr(tag))
            download_subs(sub_link)
            got_subs = True
        if not got_magnet and magnet_link is not None:
            print("    Magnet from", repr(tag))
            download_magnet(magnet_link)
            got_magnet = True


def download_cross_versions(data):
    found_sub = found_magnet = False
    for tag, magnet_link, sub_link in data:
        if magnet_link is not None and not found_magnet:
            print("Got magnet from:", repr(tag))
            download_magnet(magnet_link)
            found_magnet = True

        if sub_link is not None and not found_sub:
            print("Got sub from:   ", repr(tag))
            download_subs(sub_link)
            found_sub = True

        if found_sub and found_magnet:
            return

    if not found_magnet:
        print("Magnet not found!!")
    if not found_sub:
        print("Sub not found!!")


def main_classic(url, only_subs, version, detached, only_magnet):
    print("Getting metadata from Argenteam (only_subs={}, only_magnet={}, version={!r})".format(
        only_subs, only_magnet, version))
    all_episodes = get_data(url)

    for name, data in all_episodes:
        print("\nDownloading data for", repr(name))
        data.sort(key=sort_by_tag)
        download_episode(data, version, detached, only_subs, only_magnet)


def main_cross_versions(url):
    print("Getting metadata from Argenteam -- cross versions mode")
    all_episodes = get_data(url)

    for name, data in all_episodes:
        print("\nDownloading data for", repr(name))
        data.sort(key=sort_by_tag)
        download_cross_versions(data)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("url")
    parser.add_argument("--only-subs", action='store_true')
    parser.add_argument("--only-magnet", action='store_true')
    parser.add_argument("--detached", action='store_true')
    parser.add_argument("--classic", action='store_true')
    parser.add_argument("--version")
    args = parser.parse_args()

    if args.classic:
        main_classic(args.url, args.only_subs, args.version, args.detached, args.only_magnet)
    else:
        main_cross_versions(args.url)
